package net.plil.clubinfo.etunicorn.app.personne;

import android.app.DatePickerDialog;
import android.app.Dialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.support.design.widget.TextInputLayout;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ProgressBar;
import android.widget.Spinner;
import android.widget.Toast;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;
import com.google.gson.Gson;

import net.plil.clubinfo.etunicorn.R;
import net.plil.clubinfo.etunicorn.app.event.ModifyEvent;
import net.plil.clubinfo.etunicorn.data.Event;
import net.plil.clubinfo.etunicorn.data.Personne;
import net.plil.clubinfo.etunicorn.data.Role;
import net.plil.clubinfo.etunicorn.utils.VolleyUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Locale;


public class ModifyPersonne extends DialogFragment {

    Calendar myCalendar = Calendar.getInstance();

    EditText mCarte;
    Spinner mNaissance;
    EditText mLogin;
    Spinner mRole;
    ProgressBar mProgressBar;

    TextInputLayout mCarteInput;
    TextInputLayout mLoginInput;

    private String[] arraySpinnerNaissance;
    private ArrayAdapter<String> adapterNaissance;

    private List<Role> arraySpinnerRole;
    private ArrayAdapter<Role> adapterRole;

    public static ModifyPersonne newInstance(Personne personne) {
        ModifyPersonne f = new ModifyPersonne();

        // Supply num input as an argument.
        Bundle args = new Bundle();
        args.putSerializable("personne", personne);
        f.setArguments(args);

        return f;
    }


    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        myCalendar.set(Calendar.YEAR, myCalendar.get(Calendar.YEAR)-18);
        // Use the Builder class for convenient dialog construction
        LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.fragment_modify_personne, null);
        mLogin = (EditText) view.findViewById(R.id.modify_personne_login);
        mCarte = (EditText) view.findViewById(R.id.modify_personne_carte);
        mNaissance = (Spinner) view.findViewById(R.id.modify_personne_naissance);
        mRole = (Spinner) view.findViewById(R.id.modify_personne_role);
        mProgressBar = (ProgressBar) view.findViewById(R.id.modify_event_progress_bar);

        mLoginInput = (TextInputLayout) view.findViewById(R.id.create_personne_login_input);
        mCarteInput = (TextInputLayout) view.findViewById(R.id.create_personne_carte_input);

        final Personne personne = (Personne) getArguments().getSerializable("personne");

        mCarte.setText(personne.getIdCarte());
        mLogin.setText(personne.getLoginPoly());

        arraySpinnerRole = new ArrayList<>();
        arraySpinnerRole.add(personne.getRole());
        adapterRole = new ArrayAdapter<>(getActivity(),
                R.layout.simple_item_layout, arraySpinnerRole);
        mRole.setAdapter(adapterRole);
        changeRolePossibility();
        mNaissance = (Spinner) view.findViewById(R.id.modify_personne_naissance);
        String myFormat = "yyyy-MM-dd"; //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, Locale.FRANCE);
        if (personne.getNaissance() == null){
            arraySpinnerNaissance = new String[]{
                    getString(R.string.modify_personne_hint_date)
            };
        } else {
            arraySpinnerNaissance = new String[]{
                    sdf.format(personne.getNaissance())
            };
        }
        adapterNaissance = new ArrayAdapter<String>(getActivity(),
                R.layout.simple_item_layout, arraySpinnerNaissance);
        mNaissance.setAdapter(adapterNaissance);


        AlertDialog dialog = new AlertDialog.Builder(getActivity())
                .setTitle(R.string.modify)
                .setView(view)
                .setPositiveButton(R.string.modify, null)
                .setNegativeButton(R.string.cancel, new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyPersonne.class);
                    }
                }).create();
        // Create the AlertDialog object and return it
        dialog.setOnShowListener(new DialogInterface.OnShowListener() {
            @Override
            public void onShow(DialogInterface dialog) {
                Button button = ((AlertDialog) dialog).getButton(AlertDialog.BUTTON_POSITIVE);
                button.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        mProgressBar.setVisibility(View.VISIBLE);
                        mRole.setVisibility(View.GONE);
                        mLoginInput.setVisibility(View.GONE);
                        mCarteInput.setVisibility(View.GONE);
                        mNaissance.setVisibility(View.GONE);
                        JSONObject jsonObject = new JSONObject();
                        try {
                            jsonObject.put("carte", mCarte.getText().toString());
                            jsonObject.put("login", mLogin.getText().toString());
                            jsonObject.put("role", mRole.getSelectedItem());
                            if (!mNaissance.getSelectedItem().equals(getString(R.string.modify_personne_hint_date)))
                                jsonObject.put("naissance", mNaissance.getSelectedItem().toString());
                        } catch (JSONException e){
                            e.printStackTrace();
                        }
                        JsonObjectRequest jsonObjectRequest = new JsonObjectRequest(Request.Method.PUT, VolleyUtils.baseUri + "/personne/" + personne.getIdPersonne() ,jsonObject , new Response.Listener<JSONObject>() {
                            @Override
                            public void onResponse(JSONObject response) {
                                mProgressBar.setVisibility(View.GONE);
                                Toast.makeText(getContext(), R.string.modify_done, Toast.LENGTH_LONG).show();
                                dismiss();
                            }
                        }, new Response.ErrorListener() {
                            @Override
                            public void onErrorResponse(VolleyError error) {
                                mProgressBar.setVisibility(View.GONE);
                                Toast.makeText(getContext(), R.string.modify_refused, Toast.LENGTH_LONG).show();

                            }
                        }
                        );
                        jsonObjectRequest.setTag(ModifyPersonne.class);
                        VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonObjectRequest);
                    }
                });
            }
        });
        mNaissance.setOnTouchListener(new View.OnTouchListener() {
            @Override
            public boolean onTouch(View v, MotionEvent event) {
                if(event.getAction() == MotionEvent.ACTION_UP) {
                    new DatePickerDialog(getContext(), date, myCalendar
                            .get(Calendar.YEAR), myCalendar.get(Calendar.MONTH),
                            myCalendar.get(Calendar.DAY_OF_MONTH)).show();
                }
                return true;
            }
        });
        dialog.setCanceledOnTouchOutside(false);
        return dialog;
    }

    DatePickerDialog.OnDateSetListener date = new DatePickerDialog.OnDateSetListener() {

        @Override
        public void onDateSet(DatePicker view, int year, int monthOfYear,
                              int dayOfMonth) {
            myCalendar.set(Calendar.YEAR, year);
            myCalendar.set(Calendar.MONTH, monthOfYear);
            myCalendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
            updateLabel();
        }
    };

    private void updateLabel() {
        String myFormat = "yyyy-MM-dd"; //In which you need put here
        SimpleDateFormat sdf = new SimpleDateFormat(myFormat, Locale.FRANCE);
        arraySpinnerNaissance[0] = sdf.format(myCalendar.getTime());
        adapterNaissance.notifyDataSetChanged();
    }


    @Override
    public void onStop() {
        VolleyUtils.getInstance(getContext()).getRequestQueue().cancelAll(ModifyPersonne.class);
        dismissAllowingStateLoss();
        super.onStop();
    }

    private void changeRolePossibility(){
        JsonObjectRequest jsonObjectRequest = new JsonObjectRequest(Request.Method.GET, VolleyUtils.baseUri + "/role/" ,null , new Response.Listener<JSONObject>() {
            @Override
            public void onResponse(JSONObject response) {
                JSONArray jsonArray = null;
                try {
                    //TODO change this by the real name of the array
                    jsonArray = response.getJSONArray("roles");
                    arraySpinnerRole.clear();
                    for (int i =0; i<jsonArray.length();++i){
                        arraySpinnerRole.add(new Gson().fromJson(String.valueOf(jsonArray.getJSONObject(i)), Role.class));
                    }
                    adapterRole.notifyDataSetChanged();
                } catch (JSONException e) {
                    e.printStackTrace();
                }

            }
        }, new Response.ErrorListener() {
            @Override
            public void onErrorResponse(VolleyError error) {
                Toast.makeText(getContext(), "Can't update role", Toast.LENGTH_LONG).show();
            }
        }
        );
        jsonObjectRequest.setTag(ModifyPersonne.class);
        VolleyUtils.getInstance(getContext()).addToRequestQueue(jsonObjectRequest);
    }
}
